import re

from .source import get_source
from .exceptions import RegistrationError



def register(config, path):
    # TODO: select registration scheme (config, path)
    source = get_source(config, path)
    scheme = select_registation_scheme(config, path)
    context = scheme.get_context(source, path)

    for pre_handler in get_pre_handlers(config):
        pre_handler(config, path, context)

    for backend in get_backends(config):
        if backend.exists(source, context):
            if config.replace:
                backend.register(source, context, replace=True)
            else:
                raise RegistrationError(f'Object {context} is already registered')
        else:
            backend.register(source, context, replace=False)

    for post_handler in get_post_handlers(config):
        post_handler(config, path, context)


def select_registation_scheme(config, path):
    cfg_schemes = config['schemes']
    for cfg_scheme in cfg_schemes:
        if cfg_scheme['filter']:
            if re.match(cfg_scheme['filter'], path):
                break
        else:
            break
    else:
        # no source found
        raise RegistrationError(f'Could not find a suitable scheme for the path {path}')


def get_pre_handlers(config):
    pass


def get_post_handlers(config):
    pass


def get_backends(config):
    pass
